import "./styles.sass";
import {StateContext} from "@/common/contexts/StateContext";
import {SocketContext} from "@/common/contexts/SocketContext";
import {useContext, useState, useEffect, useRef} from "react";
import MusicSlider from "@/pages/Game/components/MusicSlider";
import {FontAwesomeIcon} from "@fortawesome/react-fontawesome";
import {faMessage} from "@fortawesome/free-solid-svg-icons";

export const Game = () => {
    const {setCurrentState} = useContext(StateContext);
    const {send, on, socket} = useContext(SocketContext);
    const [messages, setMessages] = useState([]);
    const [inputValue, setInputValue] = useState("");
    const [connectedUsers, setConnectedUsers] = useState([]);
    const [username, setUsername] = useState("");
    const messageEndRef = useRef(null);

    useEffect(() => {
        const handleChatMessage = (messageData) => {
            setMessages(prev => [...prev, messageData]);
        };
        
        const handleUserConnected = (userData) => {
            setMessages(prev => [...prev, {
                system: true,
                text: `${userData.name} ist beigetreten`
            }]);
            setConnectedUsers(prev => [...prev, userData]);
        };
        
        const handleUserDisconnected = (userId) => {
            setConnectedUsers(prev => {
                const user = prev.find(u => u.id === userId);
                if (user) {
                    setMessages(prevMsgs => [...prevMsgs, {
                        system: true,
                        text: `${user.name} hat den Raum verlassen`
                    }]);
                }
                return prev.filter(u => u.id !== userId);
            });
        };

        if (socket && socket.id) {
            send("get-user-info");
        }
        
        const handleUserInfo = (userInfo) => {
            setUsername(userInfo.name);
        };
        
        const cleanupChatMessage = on("chat-message", handleChatMessage);
        const cleanupUserConnected = on("user-connected", handleUserConnected);
        const cleanupUserDisconnected = on("user-disconnected", handleUserDisconnected);
        const cleanupUserInfo = on("user-info", handleUserInfo);
        
        return () => {
            cleanupChatMessage();
            cleanupUserConnected();
            cleanupUserDisconnected();
            cleanupUserInfo();
        };
    }, [on, send, socket]);

    useEffect(() => {
        messageEndRef.current?.scrollIntoView({behavior: "smooth"});
    }, [messages]);

    const handleSendMessage = () => {
        if (inputValue.trim()) {
            const messageData = {
                text: inputValue,
                sender: username
            };
            send("send-message", messageData);
            setMessages(prev => [...prev, messageData]);
            setInputValue("");
        }
    };

    return (
        <div className="game-page">
            <div className="background-overlay">
                <div className="rotating-gradient"></div>
            </div>
            
            <div className="main-content">
                <div className="song-display">
                    <h2>ToneGuessr</h2>
                    <div className="song-card">
                        <img src="https://mir-s3-cdn-cf.behance.net/project_modules/1400/fe529a64193929.5aca8500ba9ab.jpg" alt="Song"/>
                        <div className="song-info">
                            <div className="song-names">Black Steam</div>
                            <div className="song-description">von Carrot Quest GmbH</div>
                        </div>
                    </div>
                </div>
                <div className="chat-window">
                    <div className="chat-header">
                        <FontAwesomeIcon icon={faMessage} />
                        <div className="chat-title">Chat</div>
                    </div>
                    <div className="chat-messages">
                        {messages.map((message, index) => (
                            <div key={index} className={`message ${message.system ? 'system-message' : ''}`}>
                                {message.system ? (
                                    <span className="message-text system">{message.text}</span>
                                ) : (
                                    <>
                                        <span className="message-sender">{message.sender}:</span>
                                        <span className="message-text">{message.text}</span>
                                    </>
                                )}
                            </div>
                        ))}
                        <div ref={messageEndRef}></div>
                    </div>
                    <div className="chat-input">
                        <input type="text" value={inputValue} 
                               onChange={(e) => setInputValue(e.target.value)}
                               onKeyPress={(e) => e.key === 'Enter' && handleSendMessage()}
                               placeholder="Gib eine Nachricht ein..."
                        />
                        <button onClick={handleSendMessage}>Send</button>
                    </div>
                </div>
            </div>
            <MusicSlider/>
        </div>
    );
}